from flanautils import FlanaEnum


class WeatherEmoji(FlanaEnum):
    ZOOM_IN = '🔎➕'
    ZOOM_OUT = '🔎➖'
    LEFT = '⬅'
    RIGHT = '➡'
    TEMPERATURE = '🌡'
    TEMPERATURE_FEEL = '🖐🌡'
    CLOUDS = '☁☁'
    VISIBILITY = '〰👀'
    UVI = '↯☀'
    HUMIDITY = '💦'
    PRECIPITATION_PROBABILITY = '🎲☔'
    PRECIPITATION_VOLUME = '🪣🌧'
    PRESSURE = '🌪⇊'
    WIND_SPEED = '🌬⇉'
